@@ -58,8 +58,8 @@ void eval_assign_array_direct(arena_state *s, const char *name,
 value *eval_math_plus(value *a, value *b);
 value *eval_math_minus(value *a, value *b);
 value *eval_math_mul(value *a, value *b);
-value *eval_math_div(value *a, value *b);
-value *eval_math_mod(value *a, value *b);
+value *eval_math_div(arena_state *s, value *a, value *b);
+value *eval_math_mod(arena_state *s, value *a, value *b);
 value *eval_math_pow(value *a, value *b);
 
 /*
