@@ -9,6 +9,24 @@
 
 #include "eval.h"
 
+/*
+ * compare fn values for equality
+ */
+static int fns_are_equal(value *a, value *b)
+{
+  signature *asig = a->value_u.fn_val;
+  signature *bsig = b->value_u.fn_val;
+  
+  if (asig->type != bsig->type) {
+    return 0;
+  }
+  if (asig->type == FUNCTION_TYPE_BUILTIN) {
+    return (asig->call_u.builtin_vector == bsig->call_u.builtin_vector);
+  } else {
+    return (asig->def == bsig->def);
+  }
+}
+
 /*
  * order constants
  */
@@ -83,9 +101,13 @@ static int getorder(value *a, value *b)
         }
       }
       break;
+    case VALUE_TYPE_FN:
+      if (fns_are_equal(a, b)) {
+        res = ORDER_EQUAL;
+      }
+      break;
     case VALUE_TYPE_ARRAY:
     case VALUE_TYPE_STRUCT:
-    case VALUE_TYPE_FN:
     case VALUE_TYPE_RES:
       /* can't compute, assume not equal */
       break;
