@@ -26,19 +26,27 @@ void file_init(arena_state *s)
 {
   value *val;
   
-  val = value_make_resource(RESOURCE_FILE, stdin, file_free);
+  val = value_make_resource(stdin, file_free);
   symtab_stack_add_variable(s, "stdin", val);
   value_free(val);
 
-  val = value_make_resource(RESOURCE_FILE, stdout, file_free);
+  val = value_make_resource(stdout, file_free);
   symtab_stack_add_variable(s, "stdout", val);
   value_free(val);
 
-  val = value_make_resource(RESOURCE_FILE, stderr, file_free);
+  val = value_make_resource(stderr, file_free);
   symtab_stack_add_variable(s, "stderr", val);
   value_free(val);
 }
 
+/*
+ * Get file pointer from file resource
+ */
+static void *file_get(void *data)
+{
+  return data;
+}
+
 /*
  * Open file with given mode
  *
@@ -50,6 +58,7 @@ value *file_open(arena_state *s, unsigned int argc, value **argv)
   char *filename = argv[0]->value_u.string_val.value;
   char *filemode = argv[1]->value_u.string_val.value;
   FILE *fp;
+  value *res;
 
   if (!value_str_compat(argv[0]) || !value_str_compat(argv[1])) {
     return value_make_void();
@@ -60,7 +69,19 @@ value *file_open(arena_state *s, unsigned int argc, value **argv)
     return value_make_void();
   }
   
-  return value_make_resource(RESOURCE_FILE, fp, file_free);
+  res = value_make_resource(fp, file_free);
+  res->value_u.res_val->get = file_get;
+  return res;
+}
+
+/*
+ * Check whether resource is file resource
+ */
+value *file_is_resource(arena_state *s, unsigned int argc, value **argv)
+{
+  void *type = argv[0]->value_u.res_val->release;
+  
+  return value_make_bool(type == file_free);
 }
 
 /*
@@ -73,11 +94,11 @@ value *file_open(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_seek(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
-  int pos  = argv[1]->value_u.int_val;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
+  int pos    = argv[1]->value_u.int_val;
   
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_bool(0);
   }
   
@@ -92,11 +113,11 @@ value *file_seek(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_tell(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
   long pos;
   
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_void();
   }
 
@@ -116,14 +137,14 @@ value *file_tell(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_read(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
-  int max  = argv[1]->value_u.int_val;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
+  int max    = argv[1]->value_u.int_val;
   char *buf;
   int units;
   value *res;
   
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_void();
   }
   
@@ -143,13 +164,13 @@ value *file_read(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_write(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
-  int len  = argv[1]->value_u.string_val.len;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
+  int len    = argv[1]->value_u.string_val.len;
   const char *data = argv[1]->value_u.string_val.value;
   size_t written;
   
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_void();
   }
   
@@ -169,11 +190,11 @@ value *file_write(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_setbuf(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
-  int res = -1;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
+  int res    = -1;
   
-  if (type == RESOURCE_FILE && fp) {
+  if (type == file_free && fp) {
     if (argv[1]->value_u.bool_val) {
       res = setvbuf(fp, NULL, _IOFBF, BUFSIZ);
     } else {
@@ -191,10 +212,10 @@ value *file_setbuf(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_flush(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
   
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_bool(0);
   }
   return value_make_bool(fflush(fp) == 0);
@@ -209,10 +230,10 @@ value *file_flush(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_is_eof(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
   
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_bool(1);
   }
   return value_make_bool(feof(fp));
@@ -227,10 +248,10 @@ value *file_is_eof(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_is_error(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
 
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_bool(0);
   }
   return value_make_bool(ferror(fp));
@@ -244,10 +265,10 @@ value *file_is_error(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_clearerr(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
 
-  if (type == RESOURCE_FILE && fp) {
+  if (type == file_free && fp) {
     clearerr(fp);
   }
   return value_make_void();
@@ -263,10 +284,10 @@ value *file_clearerr(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_close(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
 
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_bool(1);
   }
 
@@ -329,12 +350,12 @@ value *file_strerror(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_getc(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
   char buf[2];
   int got;
 
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_void();
   }
   got = fgetc(fp);
@@ -351,12 +372,12 @@ value *file_getc(arena_state *s, unsigned int argc, value **argv)
  */
 value *file_gets(arena_state *s, unsigned int argc, value **argv)
 {
-  int type = argv[0]->value_u.res_val->type;
-  FILE *fp = argv[0]->value_u.res_val->data;
+  void *type = argv[0]->value_u.res_val->release;
+  FILE *fp   = argv[0]->value_u.res_val->data;
   char buf[65536];
   char *res;
   
-  if (type != RESOURCE_FILE || !fp) {
+  if (type != file_free || !fp) {
     return value_make_void();
   }
   res = fgets(buf, 65536, fp);
