@@ -20,6 +20,7 @@
 /*
  * Dictionary functions
  */
+value *dict_is_resource(arena_state *s, unsigned int, value **);
 value *dict_open(arena_state *s, unsigned int, value **);
 value *dict_read(arena_state *s, unsigned int, value **);
 value *dict_write(arena_state *s, unsigned int, value **);
@@ -138,6 +139,7 @@ value *math_abs(arena_state *s, unsigned int, value **);
  * File functions
  */
 void file_init(arena_state *s);
+value *file_is_resource(arena_state *s, unsigned int, value **);
 value *file_open(arena_state *s, unsigned int, value **);
 value *file_seek(arena_state *s, unsigned int, value **);
 value *file_tell(arena_state *s, unsigned int, value **);
@@ -237,26 +239,38 @@ value *list_replicate(arena_state *s, unsigned int, value **);
  * Memory handling functions
  */
 value *mem_make_pointer(void *, int);
+value *mem_is_resource(arena_state *s, unsigned int, value **);
 value *mem_malloc(arena_state *s, unsigned int, value **);
 value *mem_calloc(arena_state *s, unsigned int, value **);
+value *mem_realloc(arena_state *s, unsigned int, value **);
 value *mem_free(arena_state *s, unsigned int, value **);
 value *mem_cnull(arena_state *s, unsigned int, value **);
 value *mem_cstring(arena_state *s, unsigned int, value **);
 value *mem_put_char(arena_state *s, unsigned int, value **);
 value *mem_put_short(arena_state *s, unsigned int, value **);
 value *mem_put_int(arena_state *s, unsigned int, value **);
+value *mem_put_float(arena_state *s, unsigned int, value **);
+value *mem_put_double(arena_state *s, unsigned int, value **);
 value *mem_put_pointer(arena_state *s, unsigned int, value **);
 value *mem_get_char(arena_state *s, unsigned int, value **);
 value *mem_get_short(arena_state *s, unsigned int, value **);
 value *mem_get_int(arena_state *s, unsigned int, value **);
+value *mem_get_float(arena_state *s, unsigned int, value **);
+value *mem_get_double(arena_state *s, unsigned int, value **);
 value *mem_get_pointer(arena_state *s, unsigned int, value **);
 value *mem_string(arena_state *s, unsigned int, value **);
 value *mem_is_rw(arena_state *s, unsigned int, value **);
 value *mem_size(arena_state *s, unsigned int, value **);
+value *mem_cpy(arena_state *s, unsigned int, value **);
+value *mem_move(arena_state *s, unsigned int, value **);
+value *mem_cmp(arena_state *s, unsigned int, value **);
+value *mem_chr(arena_state *s, unsigned int, value **);
+value *mem_set(arena_state *s, unsigned int, value **);
 
 /*
  * Foreign function interface
  */
+value *dyn_is_resource(arena_state *s, unsigned int, value **);
 value *dyn_supported(arena_state *s, unsigned int, value **);
 value *dyn_open(arena_state *s, unsigned int, value **);
 value *dyn_close(arena_state *s, unsigned int, value **);
@@ -266,6 +280,17 @@ value *dyn_call_int(arena_state *s, unsigned int, value **);
 value *dyn_call_float(arena_state *s, unsigned int, value **);
 value *dyn_call_ptr(arena_state *s, unsigned int, value **);
 
+/*
+ * PCRE functions
+ */
+void preg_init(arena_state *s);
+value *preg_is_resource(arena_state *s, unsigned int, value **);
+value *preg_supported(arena_state *s, unsigned int, value **);
+value *preg_compile(arena_state *s, unsigned int, value **);
+value *preg_match(arena_state *s, unsigned int, value **);
+value *preg_exec(arena_state *s, unsigned int, value **);
+value *preg_free(arena_state *s, unsigned int, value **);
+
 /*
  * Register functions
  */
